"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const quidproquo_webserver_1 = require("quidproquo-webserver");
const getDomainValidationOptions_1 = require("../../../logic/acm/getDomainValidationOptions");
const findMatchingCertificates_1 = require("../../../logic/acm/findMatchingCertificates");
// // TODO: Don't use Globals like this
const GLOBAL_CERT_ARN = process.env.certificateArn;
const GLOBAL_CERT_DOMAIN = process.env.certificateDomain;
const GLOBAL_CERT_REGION = process.env.certificateRegion;
const awsToQpqEventTypeMap = {
    Create: quidproquo_webserver_1.CloudflareDnsDeployEventEnum.Create,
    Update: quidproquo_webserver_1.CloudflareDnsDeployEventEnum.Update,
    Delete: quidproquo_webserver_1.CloudflareDnsDeployEventEnum.Delete,
};
const getProcessTransformEventParams = (siteDnsBase) => {
    return ({ eventParams: [event, context] }) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        // Get all the certs that need to be added to the cloud flare
        const certArns = [
            ...(yield (0, findMatchingCertificates_1.findMatchingCertificates)(GLOBAL_CERT_DOMAIN, GLOBAL_CERT_REGION)),
            GLOBAL_CERT_ARN,
        ].filter(Boolean);
        const certDomains = yield Promise.all(certArns.map((arn) => (0, getDomainValidationOptions_1.getDomainValidationOptions)(arn, GLOBAL_CERT_REGION)));
        // Remove trailing dot from keys
        const dnsEntriesWithTrimmedKeys = Object.entries(Object.assign(Object.assign({}, event.ResourceProperties.dnsEntries), certDomains.reduce((acc, certDomain) => (Object.assign(Object.assign({}, acc), certDomain)), {}))).reduce((acc, [key, value]) => {
            const trimmedKey = key.endsWith('.') ? key.slice(0, -1) : key;
            acc[trimmedKey] = value;
            return acc;
        }, {});
        const transformedEventParams = {
            siteDns: siteDnsBase,
            RequestType: awsToQpqEventTypeMap[event.RequestType],
            apiSecretName: event.ResourceProperties.apiSecretName,
            dnsEntries: dnsEntriesWithTrimmedKeys,
            oldDnsEntries: (_a = event === null || event === void 0 ? void 0 : event.OldResourceProperties) === null || _a === void 0 ? void 0 : _a.dnsEntries,
        };
        console.log('transformedEventParams', JSON.stringify(transformedEventParams, null, 2));
        return (0, quidproquo_core_1.actionResult)(transformedEventParams);
    });
};
const getProcessTransformResponseResult = (qpqConfig) => {
    // We might need to JSON.stringify the body.
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // always success
        return (0, quidproquo_core_1.actionResult)(void 0);
    });
};
const getProcessAutoRespond = (qpqConfig) => {
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // always allow
        return (0, quidproquo_core_1.actionResult)(false);
    });
};
const getProcessMatchStory = (qpqConfig) => {
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        return (0, quidproquo_core_1.actionResult)({
            src: (0, quidproquo_core_1.getServiceEntry)('cloudflare', 'cloudflare', 'onDeploy'),
            runtime: 'onDeploy',
        });
    });
};
exports.default = (qpqConfig) => {
    const [siteDnsConfig] = quidproquo_webserver_1.qpqWebServerUtils.getDnsConfigs(qpqConfig);
    return {
        [quidproquo_core_1.EventActionType.TransformEventParams]: getProcessTransformEventParams(siteDnsConfig.dnsBase),
        [quidproquo_core_1.EventActionType.TransformResponseResult]: getProcessTransformResponseResult(qpqConfig),
        [quidproquo_core_1.EventActionType.AutoRespond]: getProcessAutoRespond(qpqConfig),
        [quidproquo_core_1.EventActionType.MatchStory]: getProcessMatchStory(qpqConfig),
    };
};
