"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const awsNamingUtils_1 = require("../../../awsNamingUtils");
const deployTypeMap = {
    'UPDATE_COMPLETE': quidproquo_core_1.DeployEventStatusType.Update,
    'CREATE_COMPLETE': quidproquo_core_1.DeployEventStatusType.Create,
    'DELETE_COMPLETE': quidproquo_core_1.DeployEventStatusType.Delete,
};
// TODO: Don't use Globals like this
const GLOBAL_DEPLOY_EVENT_NAME = process.env.deployEventConfigName;
const getProcessTransformEventParams = (qpqConfig) => {
    return ({ eventParams: [event, context] }) => __awaiter(void 0, void 0, void 0, function* () {
        const status = event.detail['status-details'].status || '';
        const stackId = event.detail['stack-id'];
        const regex = /:stack\/([^\/]+)/;
        const match = stackId.match(regex);
        const stackName = match && match[1] ? match[1] : '';
        const transformedEventParams = {
            deployEventType: quidproquo_core_1.DeployEventType.Unknown,
            deployEventStatusType: deployTypeMap[status] || quidproquo_core_1.DeployEventStatusType.Unknown,
        };
        if (stackName === (0, awsNamingUtils_1.getApiStackName)(qpqConfig)) {
            transformedEventParams.deployEventType = quidproquo_core_1.DeployEventType.Api;
        }
        else if (stackName === (0, awsNamingUtils_1.getWebStackName)(qpqConfig)) {
            transformedEventParams.deployEventType = quidproquo_core_1.DeployEventType.Web;
        }
        console.log("transformedEventParams", transformedEventParams);
        return (0, quidproquo_core_1.actionResult)(transformedEventParams);
    });
};
const getProcessTransformResponseResult = (qpqConfig) => {
    // We might need to JSON.stringify the body.
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // always success
        return (0, quidproquo_core_1.actionResult)(void 0);
    });
};
const getProcessAutoRespond = (qpqConfig) => {
    return ({ transformedEventParams }) => __awaiter(void 0, void 0, void 0, function* () {
        // exit if we don't know what deploy type this is, probably another stack
        return (0, quidproquo_core_1.actionResult)(transformedEventParams.deployEventType === quidproquo_core_1.DeployEventType.Unknown ||
            transformedEventParams.deployEventStatusType === quidproquo_core_1.DeployEventStatusType.Unknown);
    });
};
const getProcessMatchStory = (qpqConfig) => {
    const deployConfig = quidproquo_core_1.qpqCoreUtils.getDeployEventConfigs(qpqConfig).find(c => c.name === GLOBAL_DEPLOY_EVENT_NAME);
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        if (!deployConfig) {
            return (0, quidproquo_core_1.actionResultError)(quidproquo_core_1.ErrorTypeEnum.NotFound, `Could not find deploy event config ${GLOBAL_DEPLOY_EVENT_NAME}`);
        }
        return (0, quidproquo_core_1.actionResult)({
            src: deployConfig.src.src,
            runtime: deployConfig.src.runtime
        });
    });
};
exports.default = (qpqConfig) => {
    return {
        [quidproquo_core_1.EventActionType.TransformEventParams]: getProcessTransformEventParams(qpqConfig),
        [quidproquo_core_1.EventActionType.TransformResponseResult]: getProcessTransformResponseResult(qpqConfig),
        [quidproquo_core_1.EventActionType.AutoRespond]: getProcessAutoRespond(qpqConfig),
        [quidproquo_core_1.EventActionType.MatchStory]: getProcessMatchStory(qpqConfig),
    };
};
