"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const quidproquo_webserver_1 = require("quidproquo-webserver");
// TODO: Don't use Globals like this
const GLOBAL_USER_DIRECTORY_NAME = process.env.userDirectoryName;
const getProcessTransformEventParams = (serviceName) => {
    return ({ eventParams: [customMessageTriggerEvent, context] }) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const transformedEventParams = {
            eventType: quidproquo_webserver_1.EmailSendEventType.VerifyEmail,
            code: customMessageTriggerEvent.request.codeParameter,
            link: customMessageTriggerEvent.request.linkParameter,
            attributes: customMessageTriggerEvent.request.userAttributes,
            username: customMessageTriggerEvent.request.usernameParameter,
        };
        switch (customMessageTriggerEvent.triggerSource) {
            case 'CustomMessage_ForgotPassword': {
                if (((_a = customMessageTriggerEvent.request.clientMetadata) === null || _a === void 0 ? void 0 : _a['userInitiated']) === 'true') {
                    transformedEventParams.eventType = quidproquo_webserver_1.EmailSendEventType.ResetPassword;
                }
                else {
                    transformedEventParams.eventType = quidproquo_webserver_1.EmailSendEventType.ResetPasswordAdmin;
                }
                break;
            }
            case 'CustomMessage_VerifyUserAttribute':
                transformedEventParams.eventType = quidproquo_webserver_1.EmailSendEventType.VerifyEmail;
                break;
        }
        return (0, quidproquo_core_1.actionResult)(transformedEventParams);
    });
};
const getProcessTransformResponseResult = (qpqConfig) => {
    // We might need to JSON.stringify the body.
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        return (0, quidproquo_core_1.actionResult)({
            subject: payload.response.subject,
            body: payload.response.body,
        });
    });
};
const getProcessAutoRespond = (qpqConfig) => {
    const userDirectoryConfig = quidproquo_core_1.qpqCoreUtils.getUserDirectories(qpqConfig).find(ud => ud.name === GLOBAL_USER_DIRECTORY_NAME);
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f;
        switch (payload.transformedEventParams.eventType) {
            case quidproquo_webserver_1.EmailSendEventType.ResetPassword:
                return (0, quidproquo_core_1.actionResult)(!((_a = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPassword) === null || _a === void 0 ? void 0 : _a.src) &&
                    !((_b = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPassword) === null || _b === void 0 ? void 0 : _b.runtime));
            case quidproquo_webserver_1.EmailSendEventType.ResetPasswordAdmin:
                return (0, quidproquo_core_1.actionResult)(!((_c = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPasswordAdmin) === null || _c === void 0 ? void 0 : _c.src) &&
                    !((_d = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPasswordAdmin) === null || _d === void 0 ? void 0 : _d.runtime));
            case quidproquo_webserver_1.EmailSendEventType.VerifyEmail:
                return (0, quidproquo_core_1.actionResult)(!((_e = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.verifyEmail) === null || _e === void 0 ? void 0 : _e.src) &&
                    !((_f = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.verifyEmail) === null || _f === void 0 ? void 0 : _f.runtime));
        }
        return (0, quidproquo_core_1.actionResult)(false);
    });
};
const getProcessMatchStory = (qpqConfig) => {
    const userDirectoryConfig = quidproquo_core_1.qpqCoreUtils.getUserDirectories(qpqConfig).find(ud => ud.name === GLOBAL_USER_DIRECTORY_NAME);
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f;
        switch (payload.transformedEventParams.eventType) {
            case quidproquo_webserver_1.EmailSendEventType.ResetPassword:
                return (0, quidproquo_core_1.actionResult)({
                    src: (_a = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPassword) === null || _a === void 0 ? void 0 : _a.src,
                    runtime: (_b = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPassword) === null || _b === void 0 ? void 0 : _b.runtime
                });
            case quidproquo_webserver_1.EmailSendEventType.ResetPasswordAdmin:
                return (0, quidproquo_core_1.actionResult)({
                    src: (_c = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPasswordAdmin) === null || _c === void 0 ? void 0 : _c.src,
                    runtime: (_d = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.resetPasswordAdmin) === null || _d === void 0 ? void 0 : _d.runtime
                });
            case quidproquo_webserver_1.EmailSendEventType.VerifyEmail:
                return (0, quidproquo_core_1.actionResult)({
                    src: (_e = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.verifyEmail) === null || _e === void 0 ? void 0 : _e.src,
                    runtime: (_f = userDirectoryConfig === null || userDirectoryConfig === void 0 ? void 0 : userDirectoryConfig.emailTemplates.verifyEmail) === null || _f === void 0 ? void 0 : _f.runtime
                });
            default:
                return (0, quidproquo_core_1.actionResultError)(quidproquo_core_1.ErrorTypeEnum.NotFound, `Email lambda not implemented for ${payload.transformedEventParams.eventType}`);
        }
    });
};
exports.default = (qpqConfig) => {
    // TODO: Make this aware of the API that we are eventing
    const serviceName = quidproquo_core_1.qpqCoreUtils.getApplicationModuleName(qpqConfig);
    return {
        [quidproquo_core_1.EventActionType.TransformEventParams]: getProcessTransformEventParams(serviceName),
        [quidproquo_core_1.EventActionType.TransformResponseResult]: getProcessTransformResponseResult(qpqConfig),
        [quidproquo_core_1.EventActionType.AutoRespond]: getProcessAutoRespond(qpqConfig),
        [quidproquo_core_1.EventActionType.MatchStory]: getProcessMatchStory(qpqConfig),
    };
};
