"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getQueueConfigSetting = void 0;
const quidproquo_core_1 = require("quidproquo-core");
const awsLambdaUtils_1 = require("../../../awsLambdaUtils");
const getQueueConfigSetting = () => {
    const queueQPQConfigSetting = JSON.parse(process.env.queueQPQConfigSetting);
    // TODO: Validate here
    return queueQPQConfigSetting;
};
exports.getQueueConfigSetting = getQueueConfigSetting;
const getProcessTransformEventParams = (qpqConfig) => {
    return ({ eventParams: [record, context] }) => __awaiter(void 0, void 0, void 0, function* () {
        return (0, quidproquo_core_1.actionResult)({
            message: {
                type: record.type,
                payload: record.payload,
            },
        });
    });
};
const getProcessTransformResponseResult = (configs) => {
    return ({ response }) => __awaiter(void 0, void 0, void 0, function* () {
        return (0, quidproquo_core_1.actionResult)(response);
    });
};
const getProcessAutoRespond = () => {
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // If we couldn't match and hasn't thrown error, we will just gracefully exit.
        return (0, quidproquo_core_1.actionResult)(!payload.matchResult.src);
    });
};
const getProcessMatchStory = (qpqConfig) => {
    const queueQPQConfigSetting = (0, exports.getQueueConfigSetting)();
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        const queueQueueProcessors = quidproquo_core_1.qpqCoreUtils.getQueueQueueProcessors(queueQPQConfigSetting.name, qpqConfig);
        const queueTypes = Object.keys(queueQueueProcessors).sort();
        // Find the most relevant match
        const matchedQueueType = queueTypes
            .map((qt) => ({
            match: (0, awsLambdaUtils_1.matchUrl)(qt, payload.transformedEventParams.message.type),
            queueType: qt,
        }))
            .find((m) => m.match.didMatch);
        if (!matchedQueueType) {
            // If we have event bus subscriptions then we don't want to error if we can't match
            // early exit will just exit gracefully
            if (queueQPQConfigSetting.eventBusSubscriptions.length > 0) {
                return (0, quidproquo_core_1.actionResult)({});
            }
            return (0, quidproquo_core_1.actionResultError)(quidproquo_core_1.ErrorTypeEnum.NotFound, `queue type not found ${payload.transformedEventParams.message.type}`);
        }
        const sourceEntry = queueQueueProcessors[matchedQueueType.queueType];
        return (0, quidproquo_core_1.actionResult)({
            src: sourceEntry.src,
            runtime: sourceEntry.runtime,
            runtimeOptions: matchedQueueType.match.params || {},
            config: matchedQueueType.queueType,
        });
    });
};
exports.default = (qpqConfig) => {
    return {
        [quidproquo_core_1.EventActionType.TransformEventParams]: getProcessTransformEventParams(qpqConfig),
        [quidproquo_core_1.EventActionType.TransformResponseResult]: getProcessTransformResponseResult(qpqConfig),
        [quidproquo_core_1.EventActionType.AutoRespond]: getProcessAutoRespond(),
        [quidproquo_core_1.EventActionType.MatchStory]: getProcessMatchStory(qpqConfig),
    };
};
