"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const awsNamingUtils_1 = require("../../../awsNamingUtils");
const getExportedValue_1 = require("../../../logic/cloudformation/getExportedValue");
const decodeValidJwt_1 = require("../../../logic/cognito/decodeValidJwt");
const getUserDirectoryReadAccessTokenActionProcessor = (qpqConfig) => {
    return ({ userDirectoryName, serviceOverride, ignoreExpiration }, session) => __awaiter(void 0, void 0, void 0, function* () {
        const region = quidproquo_core_1.qpqCoreUtils.getApplicationModuleDeployRegion(qpqConfig);
        const userPoolId = yield (0, getExportedValue_1.getExportedValue)((0, awsNamingUtils_1.getCFExportNameUserPoolIdFromConfig)(userDirectoryName, qpqConfig, serviceOverride), region);
        const authInfo = yield (0, decodeValidJwt_1.decodeValidJwt)(userPoolId, region, ignoreExpiration, session.accessToken);
        if (!authInfo || !(authInfo === null || authInfo === void 0 ? void 0 : authInfo.username)) {
            return (0, quidproquo_core_1.actionResultError)(quidproquo_core_1.ErrorTypeEnum.Unauthorized, 'Invalid accessToken');
        }
        return (0, quidproquo_core_1.actionResult)({
            userId: authInfo.userId,
            username: authInfo.username,
        });
    });
};
exports.default = (qpqConfig) => {
    return {
        [quidproquo_core_1.UserDirectoryActionType.ReadAccessToken]: getUserDirectoryReadAccessTokenActionProcessor(qpqConfig),
    };
};
