"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getQpqAttributesFromCognitoUserAttributes = exports.getCognitoUserAttributesFromQpqUserAttributes = exports.cognitoAttributeMap = void 0;
exports.cognitoAttributeMap = {
    email: 'email',
    emailVerified: 'email_verified',
    userId: 'sub',
    address: 'address',
    birthDate: 'birthdate',
    familyName: 'family_name',
    gender: 'gender',
    givenName: 'given_name',
    locale: 'locale',
    middleName: 'middle_name',
    name: 'name',
    nickname: 'nickname',
    phoneNumber: 'phone_number',
    picture: 'picture',
    preferredUsername: 'preferred_username',
    profile: 'profile',
    website: 'website',
    zoneInfo: 'zoneinfo',
};
// Flip the cognitoAttributeMap for reverse mapping
const reversedCognitoAttributeMap = Object.entries(exports.cognitoAttributeMap).reduce((acc, [key, value]) => (Object.assign(Object.assign({}, acc), { [value]: key })), {});
const getCognitoUserAttributesFromQpqUserAttributes = (userAttributes) => {
    return Object.keys(userAttributes)
        .map((key) => ({
        Name: exports.cognitoAttributeMap[key],
        Value: `${userAttributes[key]}`,
    }))
        .filter((attribute) => !!attribute.Name && !!attribute.Value);
};
exports.getCognitoUserAttributesFromQpqUserAttributes = getCognitoUserAttributesFromQpqUserAttributes;
const getQpqAttributesFromCognitoUserAttributes = (cognitoUserAttributes) => {
    // Map cognitoUserAttributes to your UserAttributes format
    const userAttributes = cognitoUserAttributes.reduce((acc, { Name, Value }) => {
        const userAttributeKey = reversedCognitoAttributeMap[Name];
        if (!userAttributeKey) {
            return acc; // If there's no matching key in your mapping, just continue with the current accumulation
        }
        if (userAttributeKey === 'emailVerified') {
            // Assume the value of 'email_verified' is a string of 'true' or 'false'
            return Object.assign(Object.assign({}, acc), { [userAttributeKey]: Value === 'true' });
        }
        return Object.assign(Object.assign({}, acc), { [userAttributeKey]: Value });
    }, {});
    return userAttributes;
};
exports.getQpqAttributesFromCognitoUserAttributes = getQpqAttributesFromCognitoUserAttributes;
