"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeLambdaByName = void 0;
const client_lambda_1 = require("@aws-sdk/client-lambda");
const createAwsClient_1 = require("../createAwsClient");
const executeLambdaByName = (functionName, region, payload, isAsync) => __awaiter(void 0, void 0, void 0, function* () {
    const lambdaClient = (0, createAwsClient_1.createAwsClient)(client_lambda_1.LambdaClient, { region });
    const encoder = new TextEncoder();
    const encodedPayload = encoder.encode(JSON.stringify(payload));
    const response = yield lambdaClient.send(new client_lambda_1.InvokeCommand({
        FunctionName: functionName,
        Payload: encodedPayload,
        InvocationType: isAsync ? 'Event' : 'RequestResponse',
    }));
    if (response.FunctionError) {
        // Get more details about the error if available
        const errorDetails = response.Payload ? new TextDecoder().decode(response.Payload) : '';
        throw new Error(`Lambda Error: ${response.FunctionError}. Details: ${errorDetails}`);
    }
    if (!isAsync && response.Payload) {
        const jsonString = new TextDecoder().decode(response.Payload);
        if (jsonString) {
            const object = JSON.parse(jsonString);
            return object;
        }
    }
    return undefined;
});
exports.executeLambdaByName = executeLambdaByName;
